Package: QuantLib
Title: QuantLib bindings for R
Version: 1.43
Date: $Date$
Maintainer: Dirk Eddelbuettel <edd@debian.org>
Author: The QuantLib Group
Description: This package provides the SWIG-generated QuantLib bindings for R
Depends: methods
SystemRequirements: QuantLib library, Boost library
License: QuantLib License
URL: https://www.quantlib.org 
