# QuantLib Python examples

[![Binder](https://mybinder.org/badge_logo.svg)](https://mybinder.org/v2/gh/lballabio/QuantLib-SWIG/binder?urlpath=lab/tree/Python/examples)

This directory contains a number of examples of using QuantLib from
Python.  They can also run as Jupyter notebooks by means of
[Jupytext](https://jupytext.readthedocs.io/).

You can try them online thanks to [Binder](https://mybinder.org/).
If you're seeing this file as a notebook, you're probably there already.
If not, you can click the "Launch Binder" badge at the top of this file.

If you want to run these examples locally, you'll need the modules listed in the
`requirements.txt` file in the `binder` folder at the root of the [QuantLib-SWIG
repository](https://github.com/lballabio/QuantLib-SWIG); to install
them, you can execute

    pip install -r requirements.txt

from that directory.
